package com.fs.app.controller;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.fs.aicall.utils.Md5Utils;
import com.fs.app.annotation.Login;
import com.fs.app.vo.LiveInfoVo;
import com.fs.app.vo.LiveVo;
import com.fs.app.websocket.bean.SendMsgVo;
import com.fs.app.websocket.service.WebSocketServer;
import com.fs.common.core.domain.BaseEntity;
import com.fs.common.core.domain.R;
import com.fs.common.core.page.TableDataInfo;
import com.fs.common.utils.bean.BeanUtils;
import com.fs.live.domain.Live;
import com.fs.live.domain.LiveMsg;
import com.fs.live.service.ILiveMsgService;
import com.fs.live.service.ILiveService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import lombok.AllArgsConstructor;
import org.apache.http.HttpRequest;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.Base64Utils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.*;


@Api("直播信息接口")
@RestController
@AllArgsConstructor
@RequestMapping(value="/app/live")
public class LiveController extends AppBaseController {

	private static final Logger log = LoggerFactory.getLogger(LiveController.class);
	private final ILiveService liveService;
	private final ILiveMsgService liveMsgService;

	@Autowired
	private WebSocketServer webSocketServer;

	@ApiOperation("直播封面信息")
	@GetMapping("/liveInfo")
	@ApiResponse(code = 200, message = "", response = LiveInfoVo.class)
	public R liveInfo(Long id) {
		LocalDateTime now = LocalDateTime.now();
		Live live = liveService.selectLiveByLiveId(id);
		if(live == null) return R.error("未找到直播");
		if(live.getIsShow() == 2) return R.error("直播未开放");
		LiveInfoVo vo = new LiveInfoVo();
		BeanUtils.copyProperties(live, vo);
		vo.setId(live.getLiveId());
		return R.ok().put("data", vo);
	}

	@Login
	@ApiOperation("直播间")
	@GetMapping("/live")
	@ApiResponse(code = 200, message = "", response = LiveVo.class)
	public R live(Long id) {
		LocalDateTime now = LocalDateTime.now();
		Live live = liveService.selectLiveByLiveId(id);
		if(live == null) return R.error("未找到直播");
		if(live.getIsShow() == 2) return R.error("直播未开放");
		LiveVo liveVo = new LiveVo();
		BeanUtils.copyProperties(live, liveVo);
		liveVo.setNowDuration(200L);
		if(live.getStatus() == 2){
			long seconds = live.getStartTime().until(now, ChronoUnit.SECONDS);
			liveVo.setNowDuration(seconds);
		}
//		if(liveVo.getNowDuration() != null){
//			liveVo.setNowPri(BigDecimal.valueOf(liveVo.getDuration()).divide(BigDecimal.valueOf(liveVo.getNowDuration()), 20, RoundingMode.UP));
//		}
		return R.ok().put("data", liveVo);
	}

	@Login
	@ApiOperation("直播间列表")
	@GetMapping("/liveList")
	@ApiResponse(code = 200, message = "", response = LiveInfoVo.class)
	public TableDataInfo liveList() {
		return getDataTable(liveService.liveList());
	}


	@Login
	@ApiOperation("聊天记录（最新30条）")
	@GetMapping("/msgList")
	@ApiResponse(code = 200, message = "", response = LiveInfoVo.class)
	public R msgList(Long id) {
		List<LiveMsg> list = liveMsgService.list(new QueryWrapper<LiveMsg>().eq("live_id", id).orderByDesc("create_time").last(" limit 30"));
		list.sort(Comparator.comparing(BaseEntity::getCreateTime));
		return R.ok().put("data", list);
	}

	@PostMapping("/create")
	public R createRoom(@RequestBody Map<String, Object> payload) {
		if (!payload.containsKey("liveId")) {
			return R.error("直播间id缺失");
		}
		Long liveId = Long.valueOf(payload.get("liveId").toString());
		return liveService.createLiveRoom(liveId);
	}


	@PostMapping("/checkLive")
	public R checkLiving(@RequestBody Map<String, String> payload) {
		// 销售端判断是否拥有直播权限
		return liveService.checkLiving(payload);
	}

	@GetMapping("/{liveId}")
	public R getRoom(@PathVariable String liveId) {
		return liveService.getLiveRoom(liveId);
	}


	@PostMapping("/checkLiving")
	public R checkLiving(HttpServletRequest request, @RequestBody  Map<String, String> params) {
		// srs直播流服务器 判断是否拥有各种权限
		String action = params.get("action");
		if ("on_publish".equals(action)) {
			System.out.println("Handling on_publish event for stream: " + params.get("stream"));
			Map<String, String> urlParamsMap = parseQueryParams(params.get("param"));
			if (urlParamsMap.isEmpty() || urlParamsMap.get("timestamp") == null || urlParamsMap.get("sign") == null) {
				return R.error();
			}
			String s = urlParamsMap.get("timestamp");
			long timestamp = Long.parseLong(s);
			String sign = urlParamsMap.get("sign");
			Date now = new Date();
			if (((now.getTime() / 1000) - timestamp) > 5 * 60 ) {
				log.error("直播时间戳过期");
				return R.error();
			}
			String currentSign = Base64Utils.encodeToString(Md5Utils.md5(params.get("stream") + params.get("stream") + timestamp + timestamp)).replace("/","");
			if (!currentSign.equals(sign)) {
				return R.error();
			}
			Live live = liveService.selectLiveByLiveId(Long.valueOf(params.get("stream")));
			if (live.getStatus() != 2) {
				return R.error("直播未开始");
			}
			SendMsgVo sendMsgVo = new SendMsgVo();
			sendMsgVo.setMsg("开始直播");
			sendMsgVo.setCmd("live_start");
			webSocketServer.broadcastMessage(Long.valueOf(params.get("stream")), JSONObject.toJSONString(R.ok().put("data",sendMsgVo)));
			// 在这里添加你的推流校验逻辑
		} else if ("on_play".equals(action)) {
			System.out.println("Handling on_play event for stream: " + params.get("stream"));
			// 在这里添加你的播放校验逻辑
		}
		R ok = R.ok();
		// 允许连接的状态码必须是0
		ok.put("code", 0);
		// 返回 0 OK 表示允许连接
		return ok;
	}

	public static Map<String, String> parseQueryParams(String url) {
		Map<String, String> resultMap = new HashMap<>();
		if (url == null || !url.contains("?")) {
			return resultMap;
		}
		String[] queryParams = url.substring(url.indexOf('?') + 1).split("&");
		if (queryParams.length != 2) {
			return resultMap;
		}
		for (String param : queryParams) {

			String[] keyValue = param.split("=",2);
			if (keyValue.length == 2) {
				resultMap.put(keyValue[0], keyValue[1]);
			}
		}

		return resultMap;
	}
}
